;+
; NAME:
;
;   PLOTGRID
;
; VERSION:
;
;   1.0 (March 1, 2008)
;
; PURPOSE:
;
;   Plot a grid of MAPPINGS III shock models on a line ratio diagram.
;   This is the plotting program called by the SHOCKPLOT widget.   
;
; AUTHOR:
;
;   Mark G. Allen
;   Observatoire de Strasbourg
;   11 Rue de l'Universite
;   Strasbourg 67000, FRANCE
;   E-mail: allen@astro.u-strasbg.fr
;
; CATEGORY:
;
;   Theoretical Model Plotting
;
; CALLING SEQUENCE:
;
;      plot_grid,LINE1=[1432,1435],LINE2=1434,LINE3=1400,LINE4=1393, $
;                MODEL_SET_STRUCT=m_sets_struct,vra=[200,5000],      $
;                xra=[-3,3],yra=[-3,3],                              $
;                /auto_title,vlabs=[0],blabs=[0]
;   
;
; INPUTS:
;
;    ++ LINE1, LINE2, LINE3, LINE4
;
;       The emission line indexes for Y1/Y2 on the vertical axis
;       and X1/X2 on the horizontal axis:
;      
;       Y1/Y2 versus X1/X2 -> LINE3/LINE4 versus LINE1/LINE2 
;
;       The inputs for LINE1,LINE2,LINE3,LINE4 are indicies of the 
;       emission lines in the full line list array. These inputs may
;       be a single index, or an integer array of the indexes which will
;       result in the sum of the emission lines of those indicies.
;
;    ++ MODEL_SET_STRUCT
;
;       A structure of structures of the form { m0:M_n1_b0_5_s, m1:M_n1_b0_s,
;       m2:M_n1_b10_s} where the individual structures are those from the
;       Mapping Shock Model Library IDL save files in the save_files/
;       directory
; 
; KEYWORD PARAMETERS:
;
;    ++ VRA - bounding range of the shock velocities to be plotted
;       e.g. [200,1000]
;
;    ++ VLABS - Flag to label lowest B field velocity sequence with the
;               velocity
;
;    ++ xfid - array (or single value) to shift the x-coordinate of the
;              velocity labels
;
;    ++ BLABS - Flag to label lowest V mag sequence with the magnetic field
;
;    ++ AUTOTITLE - Create axis labels automatically. It constructs the axis
;                   names from the wavelengths, atom and ionization stage
;                   listed in the save files. It assusmes all the lines are
;                   from the same species. This is usually disabled for making
;                   publication plots where better axis labels can be inserted
;                   via use of the xtitle and ytitle keywords to plot that get
;                   passed through the _EXTRAS keyword
;
;    ++ P_GRID - Print the values (y-axis and x-axis) of the grid to the IDL
;                prompt. This is organised into a series of velocity sequences. 
;
;    ++ VEL_LIM - limit the number grid lines of constant velocity that are
;                 plotted to every 50 km/s (rather than every 25 km/s as
;                 available)
;
;    ++ BCTAB - the colour table to use for the grid lines of constant
;               magnetic field. If not set, or 0, then the grey scale table   
;               is used. The default used in shockplot is 1
;
;    ++ VCTAB - the colour table to use for the grid lines of constant
;               shock velocity. If not set, or 0, then the grey scale table   
;               is used. The default used in shockplot is 3
;
;    ++ CLIP_BCT - clip values for the BCTAB colour table, so to avoid highest
;                  and lowest parts of the table. A 2 -element integer array with
;                  values between 1 and 255
;
;    ++ CLIP_VCT - clip values for the BCTAB colour table, so to avoid highest
;                  and lowest parts of the table. A 2 -element integer array with
;                  values between 1 and 255
;
;    ++ LAB_COL - optional colour for the B and V labels. A two element array 
;                 [V labels colour, B labels Color]
;                 ** Waring this is not device independent **
;
;    ++ NCT  - No Colour Tables. This over-rides any colour table selection
;              and does the plot in simple black
;

;    Required parameters that are passed via the _EXTRAS keyword to the IDL
;    plot procedure:
;
;    ++ XRA, YRA, POS
;
;
;
; EXAMPLE:
;       
;        Here is an example of usage, where the files are restored, structures
;        built and then the grids plotted with plot_grid. Shockplot has the 
;        option to print out such a list of IDL prompt commands via the 
;        'Options->Print IDL Command' button
; 
;  ; ===========================================================================
;        ;
;        ;  The following commands may be entered at the IDL command
;        ;  line to reproduce this plot
;        ;
;        ;  X: NII 6547.9600 6583.3400 / HI 6562.8000
;        ;  Y: OIII 5006.7700 / HI 4861.3200
;        ;
;        ; Restore models from IDL save files
;        ; (each save file contains a structure of the same name)
;        restore, filename='save_files/M_n1_b0_5_s.save'
;        restore, filename='save_files/M_n1_b0_s.save'
;        restore, filename='save_files/M_n1_b10_s.save'
;        restore, filename='save_files/M_n1_b1_s.save'
;        restore, filename='save_files/M_n1_b2_s.save'
;        restore, filename='save_files/M_n1_b4_s.save'
;        restore, filename='save_files/M_n1_b5_s.save'
;        restore, filename='save_files/M_n1_be_s.save'
;        ;
;        ; Construct a single structure of the restored model structures
;        m_sets_struct = { m0:M_n1_b0_5_s, m1:M_n1_b0_s, m2:M_n1_b10_s,$
;                          m3:M_n1_b1_s,   m4:M_n1_b2_s, m5:M_n1_b4_s, $
;                          m6:M_n1_b5_s,   m7:M_n1_be_s}
;        ;
;        ; Call the plot_grid procedure to make the plot
;        plot_grid,LINE1=[1432,1435],LINE2=1434,LINE3=1400,LINE4=1393, $
;                        MODEL_SET_STRUCT=m_sets_struct,vra=[200,5000],$
;                        xra=[-3,3],yra=[-3,3],/auto_title,vlabs=[0],  $
;                        blabs=[0],lab_col=[255,16776960]
;        ;
; ===========================================================================
;
; OUTPUT:
;
;    Prints the grid values to the IDL console when P_GRID is set
;
; COMMON BLOCKS:
; 
;    None
;
; RESTRICTIONS:
;
;   Requires all the programs in the SHOCKPLOT package. 
;   See SHOCKPLOT restrictions.
;
;   Requires all the model data file in the save_files/ directory
;
;
; MODIFICATION HISTORY:
;
;  2006-03-30 PLOT_GRID - early days...
;  2008-03-04 PLOT_GRID - cleaned up for SHOCKPLOT v1.0

;
;##############################################################################
;
; LICENSE
;
;  Copyright (C) 2008 M. G. Allen
;
;  SHOCKPLOT is free software; you can redistribute it and/or modify it.
; 
;  Those components from other authors as listed above have licenses indicated
;  in the indivdual program files.
;
;
;##############################################################################

PRO plot_grid, LINE1=a, LINE2=b, LINE3=d, LINE4=e, MODEL_SET_STRUCT=model_set_struct, VLABS=VLABS, BLABS=BLABS, xfid=xfid, _EXTRA=extras, vra=vra, AUTO_TITLE=auto_title, P_GRID=p_grid, LAB_COL=lab_col, vel_lim=vel_lim, bctab=bctab, vctab=vctab, clip_bct=clip_bct, clip_vct=clip_vct, nct=nct, plot_ct=plot_ct

; nct - if set then no colour tables are used


x_arr = FLTARR(50,50)
y_arr = FLTARR(50,50)
v_arr = FLTARR(50,50)
b_arr = FLTARR(50,50)
model_name_arr = STRARR(50)

FOR i=0,n_tags(model_set_struct)-1 DO BEGIN
    model = model_set_struct.(i)
    get_v_seq, LINE1=a, LINE2=b, LINE3=d, LINE4=e, MODEL=model, x_out=x_out, y_out=y_out, v_out=v_out, b_out=b_out
    x_arr[i,0:n_elements(v_out)-1] = x_out
    y_arr[i,0:n_elements(v_out)-1] = y_out
    v_arr[i,0:n_elements(v_out)-1] = v_out
    b_arr[i,0:n_elements(v_out)-1] = b_out
    model_name_arr[i] = model.name
ENDFOR

v = model.v
atom = model.L.A
ion_state= model.L.I
w = model.L.W

IF keyword_set(bctab) THEN bctable = bctab ELSE  bctable = 0 ;1
IF keyword_set(vctab) THEN vctable = vctab ELSE  vctable = 0 ;3
IF keyword_set(clip_bct) THEN clip_bct_val = clip_bct ELSE  clip_bct_val = [100,200]
IF keyword_set(clip_vct) THEN clip_vct_val = clip_vct ELSE  clip_vct_val = [80,200]


;
; Construct the axis titles.
a_wave ='' 
FOR i=0,N_ELEMENTS(a)-1 DO BEGIN
a_wave = a_wave + ' ' +STRCOMPRESS(STRING(w[a[i]]), /REMOVE_ALL)
ENDFOR

b_wave ='' 
FOR i=0,N_ELEMENTS(b)-1 DO BEGIN
b_wave = b_wave + ' ' +STRCOMPRESS(STRING(w[b[i]]), /REMOVE_ALL)
ENDFOR

d_wave ='' 
FOR i=0,N_ELEMENTS(d)-1 DO BEGIN
d_wave = d_wave + ' ' +STRCOMPRESS(STRING(w[d[i]]), /REMOVE_ALL)
ENDFOR

e_wave ='' 
FOR i=0,N_ELEMENTS(e)-1 DO BEGIN
e_wave = e_wave + ' ' +STRCOMPRESS(STRING(w[e[i]]), /REMOVE_ALL)

ENDFOR

; Loop over all the model sets and plot each velocity sequence --------
;

b_values=[0.0]
FOR j = 0,n_tags(model_set_struct)-1 DO BEGIN
 b_val = b_arr[j,0]
 b_values = [b_values,b_val]
ENDFOR
b_values = b_values[1:*]

IF NOT keyword_set(nct) THEN BEGIN
set_colors, bctable, n_elements(b_values), c_indices, clip=clip_bct_val
color_vec = c_indices
sort_b = sort(b_values)
sort_sort_b = sort(sort_b)
color_vec = color_vec[sort_sort_b]
;print, b_values
;print, color_vec
ENDIF


FOR j = 0,n_tags(model_set_struct)-1 DO BEGIN
    v_vec = v_arr[j,*]
    b_vec = b_arr[j,*]
    x_vec = x_arr[j,*]
    y_vec = y_arr[j,*]
    model_name=model_name_arr[j]
    v_ind = where(v_vec NE 0)

    b_min=min(b_arr[where(b_arr GT 0.0)])
    b_max=max(b_arr)

    ; set minimum velocity for PRECUSOR models to be 200 km/s
    IF (strpos(model_name,'PRECURSOR') NE -1) AND vra[0] LT 200 THEN vra[0]=200

    IF keyword_set(vra) THEN BEGIN
    ;print, 'vra set'
    v_ind = where((v_vec NE 0) AND (v_vec GE vra[0]) AND (v_vec LE vra[1]))
    ENDIF

    x_vec2 = x_vec[v_ind]
    y_vec2 = y_vec[v_ind]
    v_vec2 = v_vec[v_ind]
    b_vec2 = b_vec[v_ind]

    IF keyword_set(auto_title) THEN BEGIN
        x_title = atom[a[0]]+ion_state[a[0]]+a_wave+' / '+atom[b[0]]+ion_state[b[0]]+b_wave
        y_title = atom[d[0]]+ion_state[d[0]]+d_wave+' / '+atom[e[0]]+ion_state[e[0]]+e_wave
        x_title_save = x_title
        y_title_save = y_title
    ENDIF ELSE BEGIN
        x_title='' 
        y_title=''
    ENDELSE


; Set line thickness depending on the device
    IF !D.name EQ 'PS' THEN thick_val = 5 ELSE thick_val=2

    ; Plot the velocity sequence
    ;
    ; 
    ;
    IF NOT keyword_set(nct) THEN current_color = color_vec[j] ELSE current_col=0
    ;
    extra_test = size(extras)
    if (extra_test[1] EQ 0) THEN extra2={a:1}
    if (extra_test[1] EQ 1) THEN BEGIN
        status = tag_exist(extras,'COL')
        if status then BEGIN
        extras2=extras
        extras=rem_tag(extras,'COL')
        endif
    ENDIF
  
    
     plot, x_vec2, y_vec2, xtitle=x_title, ytitle=y_title, _EXTRA=extras, $
    /xsty, /ysty, /noerase, thick=thick_val, /nodata
    oplot, x_vec2, y_vec2, _EXTRA=extras2, thick=thick_val,col=current_color
 
    ; Print out grid values 
    IF keyword_set(p_grid) THEN BEGIN
        print, ';'
        print, '; ', model_name
        print, '; Y = ', y_title_save
        print, '; X = ', x_title_save
        print, ';  B (10^-6 G)     V (km s^-1)   log10(X)       log10(Y)           '
        print, '; -------------------------------------------------------------'
        FOR zz=0,n_elements(b_vec2)-1 DO BEGIN
            print, b_vec2[zz], v_vec2[zz], x_vec2[zz], y_vec2[zz]
        ENDFOR
    ENDIF

    ; (We're still in the loop here!)
    ; Put velocity labels on the V_sequence for the lowest B
    ; This is a pretty ugly bit of code that needs to be re-written
    ; It contains some repeated code and awkward logic, but it works...
    IF b_vec2[0] EQ b_min THEN BEGIN
    ; define an integer array  with as many elements as there are  velocities
    v_vec2_indicies=INTARR(n_elements(v_vec2))
    ; set each value to -1
    v_vec2_indicies=v_vec2_indicies -1
    ; set xfiddle which fiddles the x position for the label 
    IF keyword_set(xfid) THEN xfiddle = xfid ELSE xfiddle=0.0
    ; 
    IF keyword_set(vlabs) THEN BEGIN
        ; find which grid points are outside the plot box
        ; so to avoid having lables outside the plot box
        xra = extras.xra
        yra = extras.yra
        ;
        ; Check whether vlabs is a vector
        IF n_elements(vlabs) GT 1 THEN BEGIN
            IF ((vra[0] EQ 200) AND (vlabs[1] EQ 200)) THEN vlabs=vlabs[1:*]
            IF (max(v_vec2) LT max(vlabs)) THEN vlabs=vlabs[where(vlabs LE max(v_vec2))]
            IF n_elements(vlabs) GT n_elements(v_vec2) THEN BEGIN
                print, 'One or more velocities specified do not correspond to a grid point'
            ENDIF ELSE BEGIN
                FOR xx=0,n_elements(vlabs)-1 DO v_vec2_indicies[xx] = where(v_vec2 EQ vlabs[xx])
                v_vec2_indicies = v_vec2_indicies[where(v_vec2_indicies NE -1)]
                v_vec2_vlab = v_vec2[v_vec2_indicies]
                x_vec2_vlab = x_vec2[v_vec2_indicies]
                y_vec2_vlab = y_vec2[v_vec2_indicies]
                off_plot_indicies = where( (x_vec2_vlab LT xra[0]) OR $
                                           (x_vec2_vlab GT xra[1]) OR $
                                           (y_vec2_vlab LT yra[0]) OR $
                                           (y_vec2_vlab GT yra[1]))
                vlab1 =strcompress(string(fix(v_vec2_vlab))) 
                IF off_plot_indicies[0] NE -1 THEN vlab1[off_plot_indicies] = ''
                xyouts, x_vec2_vlab+xfiddle, y_vec2_vlab, vlab1,color=lab_col[0] ; '0000FF'xL
            ENDELSE
        ENDIF ELSE BEGIN
        off_plot_indicies = where( (x_vec2 LT xra[0]) OR $
                                   (x_vec2 GT xra[1]) OR $
                                   (y_vec2 LT yra[0]) OR $
                                   (y_vec2 GT yra[1]))
        vlab2 =strcompress(string(fix(v_vec2)))
        IF off_plot_indicies[0] NE -1 THEN vlab2[off_plot_indicies] = ''
        xyouts, x_vec2+xfiddle, y_vec2, vlab2,color=lab_col[0] ; '0000FF'xL
        ENDELSE
        ENDIF
        ENDIF

ENDFOR
; End of loop over all models in the input structure ---------
tvals = findgen(n_elements(color_vec)+1)+0.5
IF keyword_set(plot_ct) THEN colorbar, ncolors=n_elements(color_vec), bottom=1, $
                             /vertical, ytickv=tvals,minor=0,divisions=n_elements(color_vec)-1, $
                             ticknames=string(b_values[sort_b]), pos=[0.9,0.50,0.95,0.85],title='B'


; Construct, and plot the grid lines of constant velocity,
; i.e. the Magnetic Field Sequences
;
; Get all the unique valid velocities in the grid  
vel = v_arr[uniq(v_arr,sort(v_arr))]
vel = vel[where(vel GT 0)]
IF keyword_set(vel_lim) THEN vel = 200 + indgen(17) * 50
; Make sure the velocities are within the selected range
IF keyword_set(vra) THEN BEGIN
vel = vel[where((vel NE 0) AND (vel GE vra[0]) AND (vel LE vra[1]))]
ENDIF

IF NOT keyword_set(nct) THEN BEGIN
set_colors, vctable, n_elements(vel), c_indices, clip=clip_vct_val
color_vec = c_indices
ENDIF

; Loop over all the velocities, and construct a B-sequence for each one
FOR p=0,n_elements(vel)-1 DO BEGIN
        b_seq_indicies = where(v_arr EQ vel[p])
        x_b_seq = x_arr[b_seq_indicies]
        y_b_seq = y_arr[b_seq_indicies]
        b_b_seq = b_arr[b_seq_indicies]
        ;
        ; sort with respect to B
        b_sort_indicies = sort(b_b_seq)
        x_b_seq_sort = x_b_seq[b_sort_indicies]
        y_b_seq_sort = y_b_seq[b_sort_indicies]
        ;
        ; Choose the color 
        IF NOT keyword_set(nct) THEN current_col = color_vec[p] ELSE current_col=0
        ; Set line thickness depending on the device
        IF !D.name EQ 'PS' THEN thick_val = 2 ELSE thick_val=1

        ; Plot the B-sequence
        oplot, x_b_seq_sort, y_b_seq_sort,_EXTRA=extras2, col=current_col, thick=thick_val
        ;
        ;
        ; Put B labels on the sequence with the lowest velocity
        IF blabs THEN BEGIN
            v_for_b_labels = 0
            IF keyword_set(vblabs) THEN  v_for_b_labels = where(vel EQ vblabs)
            IF p EQ v_for_b_labels THEN BEGIN
                b_b_seq = b_b_seq[b_sort_indicies]
                b_str_vec = STRARR(n_elements(b_b_seq))
                FOR i=0,n_elements(b_b_seq)-1 DO BEGIN
                    b_str = b_b_seq[i]
                    IF b_str GE 1 THEN b_str = strtrim(string(fix(b_str)),2) ELSE b_str = strtrim(string(b_str),2)
                    IF ((b_b_seq[i] LT 1) AND strpos(b_str, '00000') GT -1 )THEN BEGIN
                        strput, b_str, '     ', strpos(b_str, '00000')
                        b_str = strcompress(b_str,/remove_all)
                    ENDIF
                    ;print,  b_str
                    b_str_vec[i] = b_str
                ENDFOR

                ; Make sure labels do not go outside plot box
                ; Requires xra and yra to be set
                xra = extras.xra
                yra = extras.yra
                off_plot_indicies = where( (x_b_seq_sort LT xra[0]) OR $
                                           (x_b_seq_sort GT xra[1]) OR $
                                           (y_b_seq_sort LT yra[0]) OR $
                                           (y_b_seq_sort GT yra[1]))
                ;print, off_plot_indicies
                IF off_plot_indicies[0] NE -1 THEN b_str_vec[off_plot_indicies] = ''
                xyouts, x_b_seq_sort, y_b_seq_sort, b_str_vec, align=0.0, color=lab_col[1] ; 'FFFF00'xL
            ENDIF
        ENDIF
ENDFOR

tvals = findgen(n_elements(color_vec)+1)+0.5
IF keyword_set(plot_ct) THEN colorbar, ncolors=n_elements(color_vec), bottom=1, $
                             /vertical, ytickv=tvals,minor=0, $
                             divisions=n_elements(color_vec)-1, $
                             ticknames=string(vel), pos=[0.9,0.1,0.95,0.45], $
                             title='V'


;xyouts, 0.1, 0.95, model.name, /NORMAL

END ; of plot_grid.pro ------------------------------------------






